"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebhookSubscriptionTopic = exports.WebhookSubscriber = void 0;
const admin_graphql_api_utilities_1 = require("@shopify/admin-graphql-api-utilities");
const graphql_1 = require("./graphql");
Object.defineProperty(exports, "WebhookSubscriptionTopic", { enumerable: true, get: function () { return graphql_1.WebhookSubscriptionTopic; } });
const util_1 = require("./util");
const debug = require("debug")("shopify-client");
class WebhookSubscriber {
    constructor(shopifyAdmin) {
        this.shopifyAdmin = shopifyAdmin;
    }
    /**
     * Update Pub/Sub webhook subscriptions to match the given list of topics. Topics not present
     * in the list will have their subscriptions removed.
     */
    updatePubSubSubscriptions({ topics, pubSubTopic, pubSubProject, metafieldNamespaces, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const existingSubscriptions = admin_graphql_api_utilities_1.nodesFromEdges((yield this.shopifyAdmin.internalClient.webhookSubscriptions())
                .webhookSubscriptions.edges);
            debug(`found ${existingSubscriptions.length} existing subscriptions${0 < existingSubscriptions.length
                ? ` (${existingSubscriptions
                    .map((existingSubscription) => existingSubscription.topic)
                    .join(", ")})`
                : ""}`);
            // map existing topics, ensuring that each topic is configured correctly and only exists once
            // if not configured correctly, subscription is omitted
            const goodSubscriptions = Object.values(existingSubscriptions.reduce((acc, subscription) => {
                // this subscription is bad
                if ("JSON" !== subscription.format ||
                    !topics.includes(subscription.topic) ||
                    "WebhookPubSubEndpoint" !== subscription.endpoint.__typename ||
                    pubSubProject !== subscription.endpoint.pubSubProject ||
                    pubSubTopic !== subscription.endpoint.pubSubTopic ||
                    (!!metafieldNamespaces &&
                        !util_1.arrayEquals(metafieldNamespaces, subscription.metafieldNamespaces)) ||
                    (!metafieldNamespaces && 0 < subscription.metafieldNamespaces.length)) {
                    return acc;
                }
                // this subscription is good
                return Object.assign(Object.assign({}, acc), { [subscription.topic]: subscription });
            }, {}));
            // calculate which subscriptions need to be deleted
            const subscriptionsToDelete = existingSubscriptions.filter((subscription) => !goodSubscriptions.find((goodSubscription) => goodSubscription.id === subscription.id));
            debug(`found ${subscriptionsToDelete.length} subscriptions to delete`);
            // delete subscriptions
            for (const subscriptionToDelete of subscriptionsToDelete) {
                debug(`deleting subscription ${subscriptionToDelete.id} (${subscriptionToDelete.topic})`);
                yield this.shopifyAdmin.internalClient.webhookSubscriptionDelete({
                    id: subscriptionToDelete.id,
                });
            }
            // calculate which topics need to be created
            const topicsToCreate = topics.filter((topic) => !goodSubscriptions.find((goodSubscription) => goodSubscription.topic === topic));
            debug(`found ${topicsToCreate.length} topics to create${0 < topicsToCreate.length ? ` (${topicsToCreate.join(", ")})` : ""}`);
            // create subscriptions
            for (const topicToCreate of topicsToCreate) {
                debug(`creating subscription ${topicToCreate}`);
                const { pubSubWebhookSubscriptionCreate, } = yield this.shopifyAdmin.internalClient.pubSubWebhookSubscriptionCreate({
                    topic: topicToCreate,
                    webhookSubscription: {
                        pubSubTopic,
                        pubSubProject,
                        metafieldNamespaces,
                        format: graphql_1.WebhookSubscriptionFormat.Json,
                    },
                });
                if (pubSubWebhookSubscriptionCreate &&
                    pubSubWebhookSubscriptionCreate.userErrors &&
                    0 < pubSubWebhookSubscriptionCreate.userErrors.length) {
                    throw new Error(`webhook subscription create failed: ${pubSubWebhookSubscriptionCreate.userErrors
                        .map((userError) => userError.message)
                        .join(", ")}`);
                }
            }
            debug(`webhook subscriptions up to date`);
        });
    }
}
exports.WebhookSubscriber = WebhookSubscriber;
